/**
	AD9951-628v1.cpp

	Firmware pour piloter un circuit DDS : AD9850.
	par Silicium628.

	Ce fichier source "AD9850.cpp" est libre, vous pouvez le redistribuer et/ou le modifier selon
	les termes de la Licence Publique Générale GNU.
**/


#include <stdio.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "AD9850-628v1.h"


AD9850::AD9850()
{
}


void AD9850::reset()
{
	phase=0; // à voir...
	
 _delay_ms(1);
  port_AD980 |=  pin_RESET;
 _delay_ms(10);
  port_AD980 &= ~pin_RESET; // l'operateur "~" donne le complement (inverse des bits). ne pas confondre avec l'opérateur "!"
 _delay_ms(10);
}



void AD9850::impulse_clk_W()  // sur pin W_CL
{
 _delay_us(5);
  port_AD980 |=  pin_WCL;
 _delay_us(5);
  port_AD980 &= ~pin_WCL; // l'operateur "~" donne le complement (inverse des bits). ne pas confondre avec l'opérateur "!"
 _delay_us(5);
}



void AD9850::impulse_FQ_U()  // sur pin FQ_U
{
 _delay_us(5);
  port_AD980 |=  pin_FQU;
 _delay_us(5);
  port_AD980 &= ~pin_FQU; // l'operateur "~" donne le complement (inverse des bits). ne pas confondre avec l'opérateur "!"
 _delay_us(5);
}



/***********************************************************************
f_out = FTW x CLKIN / 2³² nous dit le datasheet
en en déduit :

FTW = f_out * 2³² / CLKIN
ici CLKIN = 125MHz / 2 = 62.5 MHz
125MHZ c'est la fréquence du quartz
cette fréquence est /2 en interne

FTW = f_out * (2³² / 62.5E6 ) = 68.71947674 x f_out


************************************************************************/



void AD9850::calcul_FTW(uint32_t frequence)
{
	float y;
	y = 68.71947674 * frequence;
	FTW = (uint32_t) y;
	
}



void AD9850::out_FTW()
{
// 40 bits vers AD9850; voir son datasheet	
	uint8_t n;

//envoi de la frequence
  uint32_t masque;

	masque = 1;
  for (n=0;  n<= 31; n++)   // on sort le LSB (W0) en premier
	{
		masque += masque;	// revient à x2 le masque le '1' se deplacant de droite a gauche, 32 bits en tout
		if ( (FTW & masque) != 0) {port_AD980 |=  pin_DATA;}	else { port_AD980 &= ~pin_DATA; }
		
		impulse_clk_W();
	}
	
  port_AD980 &= ~pin_DATA;				// (W32 toujours = 0)
  impulse_clk_W();

  port_AD980 &= ~pin_DATA;		// (W33 toujours = 0)
  impulse_clk_W();

  port_AD980 &= ~pin_DATA;		// (W34 = Power-Down = 0)
  impulse_clk_W();

// envoi de la phase (5 bits)

  for (n=0;  n<=4; n++)    // on sort le LSB (W35) en premier et le MSB en dernier.
	{
		masque = (1 << n);
		if (phase & masque) {port_AD980 |=  pin_DATA;} else { port_AD980 &= ~pin_DATA; }

		impulse_clk_W();
	}
// envoi impulsion FQ_UD
	impulse_FQ_U();
}










